program PrgStart;

{$R DATA.RES}

uses
  Windows,
  Types,
  TinyAppU,
  SysUtils,
  ShellAPI,
  Registry,
  Messages,
  Menus;

const
  CHelp = '-help';

type
  TParameters = record
    ProgramFile  : String;
    InstanceName : String;
    IconFile     : String;
    Parameters   : String;
  end;

  TProgramStart = class(TTinyApplication)
  private
  protected
    FClose       : TMenuItem;
    FDivClose    : TMenuItem;
    FDivInstance : TMenuItem;
    FForceClose  : Boolean;
    FInstance    : TMenuItem;
    FParameters  : TParameters;
    FPopupMenu   : TPopupMenu;
    FProcess     : THandle;
    FTerminate   : TMenuItem;

    procedure CreateMenu;
    procedure DestroyMenu;

    procedure StartApplication;
    procedure StopApplication;

    procedure TaskBarMenu(var AMessage : TMessage);
    procedure TerminateClick(Sender : TObject);

    procedure WriteToAutostart(const AActivate : Boolean);
  public
    constructor Create(const AParameters : TParameters);

    destructor Destroy; override;
  published
  end;

function GetParameters : TParameters;
var
  LIndex : LongInt;
begin
  Result.ProgramFile  := '';
  Result.InstanceName := '';
  Result.IconFile     := '';
  Result.Parameters   := '';

  if (ParamCount > 0) then
  begin
    Result.ProgramFile := ParamStr(1);

    if (ParamCount > 1) then
    begin
      Result.InstanceName := ParamStr(2);

      if (ParamCount > 2) then
      begin
        Result.IconFile := ParamStr(3);

        if (ParamCount > 3) then
        begin
          for LIndex := 4 to ParamCount do
            Result.Parameters := '"' + ParamStr(LIndex) + '" ';
        end;
      end;
    end;
  end;
end;

{ TProgramStart }

procedure TProgramStart.CreateMenu;
begin
  FPopupMenu := TPopupMenu.Create(nil);
  try
    FInstance := TMenuItem.Create(FPopupMenu);
    try
      if (Trim(FParameters.InstanceName) <> '') then
        FInstance.Caption := FParameters.InstanceName
      else
        FInstance.Caption := '(default)';

      FInstance.Enabled := false;
      FInstance.Name    := 'InstanceMenuItem';
      FInstance.OnClick := nil;
      FInstance.Visible := true;

      FPopupMenu.Items.Add(FInstance);
    except
    end;

    FDivInstance := TMenuItem.Create(FPopupMenu);
    try
      FDivInstance.Caption := '-';
      FDivInstance.Enabled := true;
      FDivInstance.Name    := 'DivInstanceMenuItem';
      FDivInstance.OnClick := nil;
      FDivInstance.Visible := true;

      FPopupMenu.Items.Add(FDivInstance);
    except
    end;

    FTerminate := TMenuItem.Create(FPopupMenu);
    try
      FTerminate.Caption := 'Terminate Application';
      FTerminate.Enabled := true;
      FTerminate.Name    := 'StartMenuItem';
      FTerminate.OnClick := TerminateClick;
      FTerminate.Visible := true;

      FPopupMenu.Items.Add(FTerminate);
    except
    end;

    FDivClose := TMenuItem.Create(FPopupMenu);
    try
      FDivClose.Caption := '-';
      FDivClose.Enabled := true;
      FDivClose.Name    := 'DivCloseMenuItem';
      FDivClose.OnClick := nil;
      FDivClose.Visible := true;

      FPopupMenu.Items.Add(FDivClose);
    except
    end;

    FClose := TMenuItem.Create(FPopupMenu);
    try
      FClose.Caption := 'Close Menu';
      FClose.Enabled := true;
      FClose.Name    := 'CloseMenuItem';
      FClose.OnClick := nil;
      FClose.Visible := true;

      FPopupMenu.Items.Add(FClose);
    except
    end;

    Tip := FParameters.InstanceName;
    if FileExists(FParameters.IconFile) then
      LoadIconFromFile(FParameters.IconFile)
    else
      LoadIconFromResourceID(100);

    TaskBarEvent := TaskBarMenu;
    PutToSystemTray;
  except
  end;
end;

procedure TProgramStart.DestroyMenu;
begin
  TakeFromSystemTray;

  FTaskBarEvent := nil;
  if (FClose <> nil) then
    FClose.Free;
  if (FDivClose <> nil) then
    FDivClose.Free;
  if (FDivInstance <> nil) then
    FDivInstance.Free;
  if (FInstance <> nil) then
    FInstance.Free;
  if (FTerminate <> nil) then
    FTerminate.Free;
  if (FPopupMenu <> nil) then
    FPopupMenu.Free;
end;

procedure TProgramStart.StartApplication;
var
   LSEInfo : TShellExecuteInfo;
begin
  if FileExists(FParameters.ProgramFile) then
  begin
    FillChar(LSEInfo, SizeOf(LSEInfo), 0);
    LSEInfo.cbSize       := SizeOf(TShellExecuteInfo);
    LSEInfo.fMask        := SEE_MASK_NOCLOSEPROCESS;
    LSEInfo.lpDirectory  := PChar(ExtractFilePath(FParameters.ProgramFile));
    LSEInfo.lpFile       := PChar(FParameters.ProgramFile);
    LSEInfo.lpParameters := PChar(FParameters.Parameters);
    LSEInfo.nShow        := SW_HIDE;
    LSEInfo.Wnd          := 0;

    if ShellExecuteEx(@LSEInfo) then
    begin
      FProcess := LSEInfo.hProcess;
      WriteToAutoStart(true);
    end;
  end;
end;

procedure TProgramStart.StopApplication;
begin
  TerminateProcess(FProcess, 0);

  if FForceClose then
    WriteToAutostart(false);
end;

constructor TProgramStart.Create(const AParameters : TParameters);
begin
  inherited Create('PrgStart_' + AParameters.InstanceName);

  FForceClose := false;
  FParameters := AParameters;
  CreateMenu;
  StartApplication;
end;

destructor TProgramStart.Destroy;
begin
  StopApplication;
  DestroyMenu;
  
  inherited Destroy;
end;

procedure TProgramStart.TaskBarMenu(var AMessage : TMessage);
var
  LMousePos : TPoint;
begin
  case AMessage.LParamLo of
    WM_RBUTTONDOWN :
    begin
      GetCursorPos(LMousePos);
      FPopupMenu.Popup(LMousePos.X, LMousePos.Y);
    end;
  end;
end;

procedure TProgramStart.TerminateClick(Sender : TObject);
begin
  FForceClose := true;

  SendMessage(FTinyApplicationHandle, CCloseTinyApplicationMessage, 0, 0);
end;

procedure TProgramStart.WriteToAutostart(const AActivate : Boolean);
var
  LRegistry : TRegistry;
begin
  LRegistry := TRegistry.Create;
  try
    LRegistry.RootKey := HKEY_LOCAL_MACHINE;

    if LRegistry.OpenKey('\SOFTWARE\Microsoft\Windows\CurrentVersion\Run\', false) then
    begin
      try
        if AActivate then
          LRegistry.WriteString('PrgStart_' + FParameters.InstanceName,
                                ParamStr(0) + ' "' + FParameters.ProgramFile + '" "' +
                                FParameters.InstanceName + '" "' + FParameters.IconFile +
                                '" ' + FParameters.Parameters)
        else
        begin
          if LRegistry.ValueExists('PrgStart_' + FParameters.InstanceName) then
            LRegistry.DeleteValue('PrgStart_' + FParameters.InstanceName);
        end;
      finally
        LRegistry.CloseKey;
      end;
    end;
  finally
    LRegistry.Free;
  end;
end;

begin
  with TProgramStart.Create(GetParameters) do
  begin
    try
      RunLoop(true);
    finally
      Free;
    end;
  end;
end.
